﻿using ProjectDumpTruck.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectDumpTruck.Drawnings;

public class DrawningTruck
{
    /// <summary>
    /// Класс-сущность
    /// </summary>
    public EntityTruck? EntityTruck { get; protected set; }

    /// <summary>
    /// Ширина окна
    /// </summary>
    private int? _pictureWidth;

    /// <summary>
    /// Высота окна
    /// </summary>
    private int? _pictureHeight;

    /// <summary>
    /// Левая координата прорисовки самосвала
    /// </summary>
    protected int? _startPosX;

    /// <summary>
    /// Верхняя координата прорисовки самосвала
    /// </summary>
    protected int? _startPosY;

    /// <summary>
    /// Ширина прорисовки самосвала
    /// </summary>
    private readonly int _drawningDumpTruckWidth = 130;

    /// <summary>
    /// Высота прорисовки самосвала
    /// </summary>
    private readonly int _drawningDumpTruckHeight = 90;

    // <summary>
    /// Координата X объекта
    /// </summary>
    public int? GetPosX => _startPosX;

    /// <summary>
    /// Координата Y объекта
    /// </summary>
    public int? GetPosY => _startPosY;

    /// <summary>
    /// Ширина объекта
    /// </summary>
    public int GetWidth => _drawningDumpTruckWidth;

    /// <summary>
    /// Высота объекта
    /// </summary>
    public int GetHeight => _drawningDumpTruckHeight;

    /// <summary> 
    /// Пустой конструктор
    /// </summary>
    private DrawningTruck()
    {
        _pictureWidth = null;
        _pictureHeight = null;
        _startPosX = null;
        _startPosY = null;

    }

    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Основной цвет</param>

    public DrawningTruck(int speed, double weight, Color bodyColor) : this()
    {
        EntityTruck = new EntityTruck(speed, weight, bodyColor);
    }

    /// <summary>
    /// Конструктор для наследников
    /// </summary>
    /// <param name="drawningDumpTruckWidth">Ширина прорисовки самосвала</param>
    /// <param name="drawningDumpTruckHeight">Высота прорисовки самосвала</param>

    protected DrawningTruck(int drawningDumpTruckWidth, int drawningDumpTruckHeight) : this()
    {
        _drawningDumpTruckWidth = drawningDumpTruckWidth;
        _pictureHeight = drawningDumpTruckHeight;
    }

    /// <summary>
    /// Установка границ поля
    /// </summary>
    /// <param name="width">Ширина поля</param>
    /// <param name="height">Высота поля</param>
    /// <returns>true - границы заданы, false - проверка не пройдена, нельзя разместить объект в этих размерах</returns>
    public bool SetPictureSize(int width, int height)
    {
        // TODO проверка, что объект "влезает" в размеры поля
        // если влезает, сохраняем границы и корректируем позицию объекта, если она была уже установлена
        if (_drawningDumpTruckWidth > width || _drawningDumpTruckHeight > height)
        {
            return false;
        }

        _pictureWidth = width;
        _pictureHeight = height;

        if (_startPosX.HasValue && _startPosX.Value + _drawningDumpTruckWidth > _pictureWidth)
        {
            _startPosX = _pictureWidth - _drawningDumpTruckWidth;
        }

        if (_startPosY.HasValue && _startPosY.Value + _drawningDumpTruckHeight > _pictureHeight)
        {
            _startPosY = _pictureHeight - _drawningDumpTruckHeight;
        }

        return true;
    }

    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (!_pictureHeight.HasValue || !_pictureWidth.HasValue)
        {
            return;
        }
        // TODO если при установке объекта в эти координаты, он будет "выходить" за границы формы
        // то надо изменить координаты, чтобы он оставался в этих границах

        if (x + _drawningDumpTruckWidth > _pictureWidth)
        {
            x = (int)_pictureWidth - _drawningDumpTruckWidth;
        }
        if (y + _drawningDumpTruckHeight > _pictureHeight)
        {
            y = (int)_pictureHeight - _drawningDumpTruckHeight;
        }
        if (x < 0)
        {
            x = 0;
        }
        if (y < 0)
        {
            y = 0;
        }

        _startPosX = x;
        _startPosY = y;

    }
    /// <summary>
    /// Изменение направления перемещения
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - перемещение выполнено, false - перемещение невозможно</returns>
    public bool MoveTransport(DirectionType direction)
    {
        if (EntityTruck == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return false;
        }

        switch (direction)
        {
            //влево
            case DirectionType.Left:
                if (_startPosX.Value - EntityTruck.Step > 0)
                {
                    _startPosX -= (int)EntityTruck.Step;
                }
                return true;

            //вверх
            case DirectionType.Up:
                if (_startPosY.Value - EntityTruck.Step > 0)
                {
                    _startPosY -= (int)EntityTruck.Step;
                }
                return true;

            //вправо
            case DirectionType.Right:
                if (_startPosX.Value + EntityTruck.Step + _drawningDumpTruckWidth < _pictureWidth)
                {
                    _startPosX += (int)EntityTruck.Step;
                }
                return true;

            //вниз
            case DirectionType.Down:
                if (_startPosY.Value + EntityTruck.Step + _drawningDumpTruckHeight < _pictureHeight)
                {
                    _startPosY += (int)EntityTruck.Step;
                }
                return true;
            default:
                return false;
        }

    }

    public virtual void DrawTransport(Graphics g)
    {
        if (EntityTruck == null || !_startPosX.HasValue || !_startPosY.HasValue)
        {
            return;
        }

        Pen pen = new(Color.Black);

        //Отрисовка основы (кабины водителя и днища)
        Brush body = new SolidBrush(EntityTruck.BodyColor);
        g.FillRectangle(body, _startPosX.Value + 100, _startPosY.Value, 30, 35);
        g.FillRectangle(body, _startPosX.Value, _startPosY.Value + 40, 130, 20); 

        //Отрисовка колёс
        Brush wheels = new SolidBrush(Color.Gray);
        g.FillEllipse(wheels, _startPosX.Value, _startPosY.Value + 60, 30, 30);
        g.FillEllipse(wheels, _startPosX.Value + 30, _startPosY.Value + 60, 30, 30);
        g.FillEllipse(wheels, _startPosX.Value + 100, _startPosY.Value + 60, 30, 30);


        //Отрисовка границ
        Brush border = new SolidBrush(Color.Black);
        g.DrawRectangle(pen, _startPosX.Value + 100, _startPosY.Value, 30, 35);
        g.DrawRectangle(pen, _startPosX.Value, _startPosY.Value + 40, 130, 20);
        g.DrawEllipse(pen, _startPosX.Value, _startPosY.Value + 60, 30, 30); 
        g.DrawEllipse(pen, _startPosX.Value + 30, _startPosY.Value + 60, 30, 30);
        g.DrawEllipse(pen, _startPosX.Value + 100, _startPosY.Value + 60, 30, 30);
    }
}
